.8086
	include	casm.inc


	mul8		MACRO	cell
		shl	cell,1
		shl	cell,1
		shl	cell,1
	ENDM
	div8		MACRO	cell
		shr	cell,1
		shr	cell,1
		shr	cell,1
	ENDM


CODE	segment	byte public
	assume cs:CODE



	dest_map_width			dw		(?)
	sour_map_width			dw		(?)
	dest_top_offs			dw		(?)
	sour_box_row_add		dw		?
	dest_box_row_add		dw		?
	col_counter			dw		?
	row_counter         	    	dw		?
	cell1				dw		?
	cell2				dw		?
	curr_di				dw		?
	byte_width			dw		?
	shrink_range			dw              ?
	height				dw		?
        grow_range                      dw              ?
        dest_map_height                 dw              ?


;***************************************************************************
;	This procedure rotates rect region in the clockwise direction
;
;	6	8	10	14            		16
;	sour		dest	sour_map_width		dest_map_width
;       14                10        8                       6


	start_c_proc	rotate_on
	cli
	lds	si,dword ptr [bp+14]		;install pointers to source
	les	di,dword ptr [bp+10]		;and destination

	push	[bp+8]				;get byte width
	pop	sour_map_width			;of source and destination

	push	[bp+6]
	pop	dest_map_width

	add	di,dest_map_width		;points to initial destination
	dec	di				;top
	mov	dest_top_offs,di

	mov	ax,sour_map_width		;get box row by row addition
	shl	ax,1
	shl	ax,1
	shl	ax,1
	sub	ax,sour_map_width
	mov	sour_box_row_add,ax

	push	dest_map_width
	pop	col_counter

	mov	cell1,sp			;store stack condition
	rot_on_column_loop:
		mov	di,dest_top_offs

		mov	ax,sour_map_width
		mov	row_counter,ax
		rot_on_row_loop:
			mov	cell2,si
			jmp	rot_8x8_on
		rot_on_next_box:
			mov	si,cell2
			inc	si		;advance to next src box in row
			dec	row_counter
		jnz	rot_on_row_loop


		dec	dest_top_offs		;advance to next top box
		add	si,sour_box_row_add	;advance to next source row
		dec	col_counter
	jnz	rot_on_column_loop

	mov	sp,cell1			;restore stack condition

	sti
	end_c_proc	rotate_on ,12


;****************************************************************************
;	This subprocedure rotates 8-byte rectangle  against  clockwise direct
;	and stores it at destination without any intermediate buffer .
;	ds:si	points to bit map 8x8 rectangle
;	es:di	points to 8x8 map at the destination
;	all registers are destroyed after this operation
;

	rot_8x8_on:
	mov	cell1,sp
	mov	sp,sour_map_width
	mov	curr_di,di

	dec	si				;decrement to fit
						;lower order byte of di

	mov	bp,8
	rot_on_col_loop:
		mov	di,[si]			;di contains current byte
		shl	di,1			;source to destination
		rcr	al,1
		shl	di,1
		rcr	ah,1
		shl	di,1
		rcr	bl,1
		shl	di,1
		rcr	bh,1
		shl	di,1
		rcr	cl,1
		shl	di,1
		rcr	ch,1
		shl	di,1
		rcr	dl,1
		shl	di,1
		rcr	dh,1
		add	si,sp
	dec	bp
	jnz     rot_on_col_loop



	mov	sp,dest_map_width
	mov	di,curr_di

	mov	es:[di],al			;update matrix	8x8
	add	di,sp
	mov	es:[di],ah
	add	di,sp
	mov	es:[di],bl
	add	di,sp
	mov	es:[di],bh
	add	di,sp
	mov	es:[di],cl
	add	di,sp
	mov	es:[di],ch
	add	di,sp
	mov	es:[di],dl
	add	di,sp
	mov	es:[di],dh
	add	di,sp				;di points to dw box accurately


	mov	sp,cell1
	jmp	rot_on_next_box

;****************************************************************************
;	This procedure shrinks bitmap buffer in vertical direction.
;	bx,di - result of shrinking of the group.
;	si    -current shrinked line in the group
;	6		10         	12    		14
;	buffer		byte width	height		shrink range
;       12                  10               8                6

	start_c_proc		shrink
	cld

	cli					;clear interrupts
	lds	bx,dword ptr [bp+12]		;get buffer start address
	push	ds
	pop	es
	mov	si,bx				;get initial shrinking addr.
	mov	di,bx

	push	[bp+10]				;get parameters
	pop	byte_width
	push	[bp+8]
	pop	height
	push	[bp+6]
	pop	shrink_range

	xor	dx,dx				;get number of lines
	mov	ax,height			;in the final  picture
	div	shrink_range
	mov	dx,ax

	update_shrink_line:                     ;update curr shrink line
	mov	di,bx				;clears result shrink line
	xor	al,al
	mov	cx,byte_width
	rep	stosb


	mov	bp,shrink_range			;get  group range
	shrink_line_group:
		mov	di,bx			;get shrink result-line address
		mov	cx,byte_width           ;shrink each byte in the row
		shrink_line_loop:               ;
			lodsb                   ;get source byte
			or	[di],al         ;shrink
			inc	di              ;advance to next byte
		loop	shrink_line_loop        ;one  more byte ?

	dec	bp				;is group updated?
	jnz	shrink_line_group		;no,shrink next.

	add	bx,byte_width			;advance to next result line
	dec	dx
	jnz	update_shrink_line

	sti
	end_c_proc		shrink,10



;****************************************************************************
;	This procedure grows bitmap buffer in vertical direction.
;	bx,di - result of shrinking of the group.
;	si    -current shrinked line in the group
;	6		10         	12    		14
;	buffer		byte width	height		shrink range

	start_c_proc		grow
	cld
	cli					;clear interrupts
	lds	bx,dword ptr [bp+12]		;get buffer start address
	push	ds
	pop	es
	mov	di,bx
	mov	si,bx

	push	[bp+10]				;get parameters
	pop	byte_width
	push	[bp+8]
	pop	height
	push	[bp+6]
	pop	grow_range

	xor	dx,dx				;get number of lines
	mov	ax,height			;for growing
	div	grow_range
	mov	bx,ax

	mov	ax,byte_width			;calculate destination row
	mul	grow_range			;addition
	mov	cx,ax

	dec	bx

	xor	dx,dx
	mov	ax,cx
	mul	bx
	add	di,ax


	xor	dx,dx
	mov	ax,byte_width
	mul	bx
	add	si,ax


	inc	bx
	mov	ax,byte_width
	mov	dx,cx
	shl	dx,1
	grow_next_line:
		mov	bp,grow_range
		grow_line:			;grow one line of bitmap
		mov	cx,ax
			rep	movsb
			sub	si,ax
		dec	bp
		jnz	grow_line

		sub	si,ax			;advance to next row
		sub	di,dx
	dec	bx
	jnz	grow_next_line

	sti
	end_c_proc		grow ,10

;***************************************************************************
;	This procedure rotates rect region in the underclockwise direction
;
;	6	8	10	14            		16
;	sour		dest	sour_map_width		sour_map_height

	start_c_proc	rotate_off
	cli
	lds	si,dword ptr [bp+14]		;install pointers to source
	les	di,dword ptr [bp+10]		;and destination

	push	[bp+8]				;get byte width
	pop	sour_map_width			;of source and destination

	mov	ax,[bp+6]
	dec	ax				;get destination map width
	div8	ax
	inc	ax
	mov	dest_map_width,ax



	mov	ax,sour_map_width		;get box row by row addition
	mul8	ax
	mov	dest_map_height,ax

	sub	ax,sour_map_width
	mov	sour_box_row_add,ax

	xor	dx,dx
	mov	ax,dest_map_height
	dec	ax
	mul	dest_map_width
	add	di,ax
	mov	dest_top_offs,di



	push	dest_map_width
	pop	col_counter

	mov	cell1,sp			;store stack condition
	rot_off_column_loop:
		mov	di,dest_top_offs

		mov	ax,sour_map_width
		mov	row_counter,ax
		rot_off_row_loop:
			mov	cell2,si
			jmp	rot_8x8_off
		rot_off_next_box:
			mov	si,cell2
			inc	si		;advance to next src box in row
			dec	row_counter
		jnz	rot_off_row_loop


		inc	dest_top_offs		;advance to next top box
		add	si,sour_box_row_add	;advance to next source row
		dec	col_counter
	jnz	rot_off_column_loop

	mov	sp,cell1			;restore stack condition

	sti
	end_c_proc	rotate_off , 12


;****************************************************************************
	rot_8x8_off:
	mov	cell1,sp
	mov	sp,sour_map_width
	mov	curr_di,di


	mov	bp,8
	rot_off_col_loop:
		mov	di,[si]			;di contains current byte
		shr	di,1			;source to destination
		rcl	al,1
		shr	di,1
		rcl	ah,1
		shr	di,1
		rcl	bl,1
		shr	di,1
		rcl	bh,1
		shr	di,1
		rcl	cl,1
		shr	di,1
		rcl	ch,1
		shr	di,1
		rcl	dl,1
		shr	di,1
		rcl	dh,1
		add	si,sp
	dec	bp
	jnz     rot_off_col_loop



	mov	sp,dest_map_width
	mov	di,curr_di

	mov	es:[di],dh			;update matrix	8x8
	sub	di,sp
	mov	es:[di],dl
	sub	di,sp
	mov	es:[di],ch
	sub	di,sp
	mov	es:[di],cl
	sub	di,sp
	mov	es:[di],bh
	sub	di,sp
	mov	es:[di],bl
	sub	di,sp
	mov	es:[di],ah
	sub	di,sp
	mov	es:[di],al
	sub	di,sp				;di points to dw box accurately


	mov	sp,cell1
	jmp	rot_off_next_box







CODE	ends

	public	rotate_on
	public	shrink
	public	rotate_off
	public	grow

END